/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

proc sql noprint;
 create table tzupdate as  
  select resInv.id as sysInfoResource,
         resInv.deleted as sysInfoDeleted,
         put(upcase(scan(resInv.name,1)),$char500.) as hostname,
         put(upcase(put(platformResource,res2name.)),$char500.) as fqdn,
         resInv.platformResource as platformResource,
         put(resInv.platformResource,RESDELETED.) as platformDeleted,
         put(prop.propvalue,$char1000.) as timezone
   from acm.resourceInventory resInv,       
        evdb.eam_resource eamRes,
        evdb.eam_cprop prop,
        evdb.eam_cprop_key pkey
    where resInv.Type eq "SAS System Info" and 
          resInv.id eq eamRes.id  and
          eamRes.instance_id eq prop.appdef_id and
          prop.keyid eq pkey.id and
          pkey.propkey eq "timezone"
    order by  platformResource desc,  sysInfoResource desc, platformDeleted, sysInfoDeleted;
quit;

data get1rec;
 set tzupdate;
 by  descending platformResource;
 if first.platformResource;
 if (platformDeleted eq "TRUE" or sysInfoDeleted eq 1) then newDelete=1;
 drop sysInfoResource sysInfoDeleted;
run;

%macro initTable;
 %if %sysfunc(exist(EVDM.PLATFORMTZ)) eq 0 %then
 %do; /* table does not exist */

data evdm.platformtz;
 length hostname $ 500 fqdn $ 500 platformResource 8 platformDeleted $5 timezone $1000; 
 stop;
run;

 %end;/* table does not exist */
%mend initTable;
%initTable;

 %lockds(lib=evdm,table=platformTZ,action=LOCK,abortOnErr=Y);

 data evdm.platformTZ;
  length origtz $1000;
  merge evdm.platformTZ(in=old) work.get1rec (in=new);
  by descending platformResource;

  if old and new then 
  do; /* matching keys */

     flag="UPDATE";     
     if deleted eq . and newDelete eq 1 then deleted=1;     
     
  end;/* matching keys */
  else if old and NOT new then 
  do; /* deleted record */

     flag="DELETE";
     if deleted eq . then deleted=1;

  end;/* deleted record */
  else if new and not old then 
  do; /* new record */

     flag="NEWREC";
     if deleted eq . and newDelete eq 1 then deleted=1;     

  end;/* new record */

  if tzoneoff(timezone) eq . then 
  do; /* original timezone is not recognized by SAS */

     origTZ=timezone;
     timezone="&evdmtz";

  end;/* original timezone is not recognized by SAS */

  if deleted eq . then deleted=0;
  drop flag newDelete;
run;

proc sort data=evdm.platformtz out=sorted;
 by  hostname  fqdn   descending platformResource  deleted timezone ;
run;

%lockds(lib=evdm,table=platformTZ,action=CLEAR);

data fmt;
 set sorted(rename=(timezone=label));
 by  hostname  fqdn   descending platformResource deleted label;
 
 keep fmtname start label;

 fmtname="$GET_TZ"; 
 
 if first.hostname then
 do; /* first hostname */

    start=hostname;
    output;

 end;/* first hostname */

 if first.fqdn then
 do; /* first fqdn */

    start=fqdn;
    output;

 end;/* first fqdn */

 if first.platformResource then
 do; /* first resource-id */

    start=put(platformResource,z6.);
    output;

 end;/* first resource-id */
run;

proc sort data=fmt out=fmt_sorted nodupkey;
 by start;
run;

data other_record;
  fmtname="$GET_TZ"; 
  start=" ";
  label="&evdmtz";
  hlo="O";
 run;

/* need to concatenate data sets to  */
/* ensure there is input data set to */
/* the proc format step.             */
 data fmt2;
  set fmt_sorted(in=a where=(label is not missing)) work.other_record(in=b);
 run;

 proc format lib=evdmfmt cntlin=work.fmt2;
run;

