/***************************************************/
/* Copyright (c) 2014 SAS Institute Inc.           */
/* SAS Campus Drive, Cary, NC, USA                 */
/* All Rights Reserved                             */
/***************************************************/

%jobheader(jobname=getMetricData);

%global _lower_ _upper_;
%getdaterange(data=evdmraw.hq_metric_data);

%put ************************************;
%put _lower_= &_lower_ _upper_= &_upper_;
%put ************************************;

filename acm_mets "&SASEVLOGDIR/metrics/acm_etl.metrics";

/* (re-)initialize record count macro vars */
%global recsBefore recsAfter;
%let recsBefore=0;
%let recsAfter=0;

%macro createhqmetricds;
	%let dsn = %str(evdmraw.hq_metric_data);
	%if NOT %sysfunc(exist(&dsn)) %then %do;
		data evdmraw.hq_metric_data;
			length datetime 8.;
			length measurement_id 8.;
			length value 8.; 
		run;
	%end;
%mend;

%createhqmetricds;

data _null_;
  set evdmraw.hq_metric_data(keep=datetime) nobs=nobs;
  call symput('recsBefore',put(nobs,best.));
  put "INFO: Table EVDMRAW.HQ_METRIC_DATA has " nobs " records at start of SAS Job getMetricData.";
  stop;
run;

data _null_;
 call symput("epochStart",put("01JAN1970:00:00"dt,best.));
run;

data _null_;
 length line $1000;
 appendLine="proc append base=evdmraw.hq_metric_data data=work.hq_metric_data;run;";
 baseDate="31DEC2005"d;
 cycleLength=9;

 /* need to handle missing start values (should */
 /* only happen on initial deployment) and/or   */ 
 /* lower bound that is further back than the   */
 /* data available.  In both cases, default to  */
 /* 9 days ago since that is the max stored in  */
 /* the SAS Environment Manager database.       */
 start=max(&_lower_,input(put(today()-8,date9.)||":00:00",datetime.));

 startDate = datepart(start) ;
 startTime = timepart(start) ;
 endDate = datepart(&_upper_);
 endTime = timepart(&_upper_) ;

 do day = startDate to endDate;

    table=mod((day-baseDate),cycleLength);

    pStart=0;
    pEnd=1;

    if day eq startDate and startTime gt sum("11:59"t,-1*&evdmtzoffset) then pStart=1; 
    if day eq endDate and endTime le sum("11:59"t,-1*&evdmtzoffset) then pEnd=0;
     
    do period = pStart to pEnd;
         
       tablename=cats("hq_metric_data_",put(table,1.),'d_',put(period,1.),'s');

       line= cats('%genextractV2(selectvars=%str(sum(timestamp/1000,&epochStart) as datetime, measurement_id, "value"),',
                                 "out=work.hq_metric_data,",
                                 'orderby=%str(timestamp, measurement_id),',
                                 "subondate=Y,",
                                 "lowerlimit=",start,",",
                                 "upperlimit=&_upper_,",
                                 "table=", tablename, ");");
       %putorex(mode=EXECUTE);
       %putorex(mode=EXECUTE, var=appendLine);

       tablecount+1;

    end; /* period loop */
 end; /* day loop */ 

 call symput('TABLECOUNT',put(tablecount,best.));

run;


/* ************************** */
/* METRICS for ACM_ETL Plugin */
/* ************************** */
 data tablerange; 
  set evdmraw.hq_metric_data(keep=datetime) nobs=nobs;
  call symput('recsAfter',put(nobs,best.));
  put "NOTE: Table EVDMRAW.HQ_METRIC_DATA has " nobs "records at end of SAS Job getMetricData.";
  stop;
run;

data _null_;
 file acm_mets mod;

 timeperiod=&_upper_-&_lower_;

 put "#Extract Time Period";
 put "extractTimePeriod=" timeperiod;

 newMetricRecords=max(0,&recsAfter-&recsBefore);
 put "#The number of new raw metric records added";
 put "newMetricRecords=" newMetricRecords;
run; 


